Package: tinytest2JUnit
Type: Package
Title: Convert 'tinytest' Output to JUnit XML
Version: 1.1.3
Maintainer: Lennart Tuijnder <lennart.tuijnder@openanalytics.eu>
Authors@R: c(
	person(given = "Anne-Katrin", family = "Hess", email = "anne-katrin.hess@openanalytics.eu", role = c("aut")),
	person(given ="Lennart", family = "Tuijnder", email = "lennart.tuijnder@openanalytics.eu", role = c("aut", "cre")))
Description: Unit testing is a solid component of automated CI/CD pipelines. 
	'tinytest' - a lightweight, zero-dependency  alternative to 'testthat' was developed.
	To be able to integrate 'tinytests' results into common CI/CD systems the 'tinytests'-object
	is converted to JUnit XML format. 'tinytest2JUnit' enables this conversion 
	while staying lightweight, having only 'tinytest' as its dependency.
Imports: tools, parallel, utils, tinytest
URL: https://github.com/openanalytics/tinytest2JUnit
BugReports: https://github.com/openanalytics/tinytest2JUnit/issues
License: GPL-3
Copyright: Open Analytics NV, 2024
RoxygenNote: 7.2.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-13 06:27:50 UTC; ltuijnder
Author: Anne-Katrin Hess [aut],
  Lennart Tuijnder [aut, cre]
Repository: CRAN
Date/Publication: 2026-04-13 06:50:02 UTC
Built: R 4.6.0; ; 2026-04-27 04:38:10 UTC; unix
