# Generated by roxygen2: do not edit by hand

S3method("[",tinytests2JUnit)
S3method(format,XMLtag)
S3method(print,XMLtag)
export(runTestDir)
export(testPackage)
export(writeJUnit)
importFrom(parallel,clusterCall)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(tinytest,run_test_file)
importFrom(tinytest,tinytest)
importFrom(tools,file_path_sans_ext)
importFrom(utils,getFromNamespace)
importFrom(utils,getSrcDirectory)
importFrom(utils,getSrcFilename)
importFrom(utils,getSrcLocation)
importFrom(utils,tail)
