% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fibonacci.R
\name{fibonacci}
\alias{fibonacci}
\title{Fibonacci}
\usage{
fibonacci(n)
}
\arguments{
\item{n}{an integer}
}
\value{
A numeric vector of length n
}
\description{
Compute first n fibonacci numbers.
}
